from cone.app.browser.actions import LinkAction
from cone.app.browser.actions import get_action_context
from cone.app.browser.utils import format_date
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.browser.utils import node_icon
from cone.app.browser.utils import node_path
from cone.app.interfaces import IWorkflowState
from cone.app.model import AppRoot
from cone.app.ugm import principal_data
from cone.app.utils import safe_decode
from cone.tile import Tile
from cone.tile import render_template
from cone.tile import render_tile
from cone.tile import tile
from node.utils import LocationIterator
from odict import odict
from pyramid.i18n import TranslationStringFactory


_ = TranslationStringFactory('cone.app')


@tile(name='logo', path='templates/logo.pt', permission='login')
class LogoTile(Tile):
    """Tile rendering the logo.
    """


@tile(name='livesearch', path='templates/livesearch.pt', permission='login')
class LivesearchTile(Tile):
    """Tile rendering the live search.
    """


@tile(name='footer', path='templates/footer.pt', permission='login')
class FooterTile(Tile):
    """Tile rendering the page footer.
    """


class ProtectedContentTile(Tile):
    """A tile rendering the loginform instead default if user is not
    authenticated.

    Normally used for 'content' tiles if page should render login form in place
    instead of throwing Unauthorized.
    """

    def __call__(self, model, request):
        if not request.authenticated_userid:
            return render_tile(model, request, 'loginform')
        return Tile.__call__(self, model, request)


@tile(name='layout', path='templates/layout.pt', permission='login')
class Layout(Tile):
    """Main layout tile.
    """

    @property
    def contenttile(self):
        return get_action_context(self.request).scope


class ViewSettingsAction(LinkAction):
    text = _('settings', default='Settings')
    icon = 'ion-ios7-gear'
    event = 'contextchanged:#layout'
    path = 'href'

    @property
    def settings(self):
        root = self.model.root
        return root and root.get('settings') or None

    @property
    def target(self):
        return make_url(self.request, node=self.settings)

    href = target

    @property
    def display(self):
        settings = self.settings
        if not settings:
            return False
        if not self.request.has_permission('view', settings):
            return False
        if not len(settings):
            return False
        return True


class LogoutAction(LinkAction):
    text = _('logout', default='Logout')
    icon = 'ion-log-out'
    action = 'logout:NONE:NONE'
    path_action = ''
    path_event = ''

    @property
    def href(self):
        return make_url(self.request, resource='logout')

    @property
    def path(self):
        return '/'


personal_tools = odict()
personal_tools['settings'] = ViewSettingsAction()
personal_tools['logout'] = LogoutAction()


@tile(name='personaltools',
      path='templates/personaltools.pt',
      permission='view',
      strict=False)
class PersonalTools(Tile):
    """Personal tool tile.
    """

    @property
    def user(self):
        userid = self.request.authenticated_userid
        data = principal_data(userid)
        fullname = data.get('fullname', userid)
        return fullname or userid

    @property
    def items(self):
        return [_(self.model, self.request) for _ in personal_tools.values()]


@tile(name='mainmenu',
      path='templates/mainmenu.pt',
      permission='view',
      strict=False)
class MainMenu(Tile):
    """Main Menu tile.

    * set ``skip_mainmenu`` on ``model.properties`` to ``True`` if node should
      not be displayed in mainmenu.

    * set ``mainmenu_display_children`` on ``model.properties`` to ``True`` if
      child nodes should be rendered as dropdown menu.

    * set ``mainmenu_empty_title`` on ``model.root.properties`` to ``True``
      if you want to render empty links in mainmenu.

    * If ``default_child`` is set on ``model.root.properties``, it is marked
      selected if no other current path is found.

    * If ``default_content_tile`` is set on ``model.root.properties``, it is
      considered in target link creation.
    """

    @property
    def menuitems(self):
        ret = list()
        path = node_path(self.model)
        if path:
            curpath = path[0]
        else:
            curpath = ''
        # work with ``self.model.root.keys()``, ``values()`` propably not works
        # due to the use of factory node.
        root = self.model.root
        root_props = root.properties
        # check for default child id if no curpath
        if not curpath and root_props.default_child:
            curpath = root_props.default_child
        # check wether to render mainmenu item title
        empty_title = root_props.mainmenu_empty_title
        # XXX: icons
        for key in root.keys():
            child = root[key]
            props = child.properties
            if self.ignore_node(child, props):
                continue
            selected = curpath == key
            item = self.create_item(child, props, empty_title, selected)
            if props.mainmenu_display_children:
                item['children'] = self.create_children(child, selected)
            else:
                item['children'] = None
            ret.append(item)
        return ret

    def create_children(self, node, selected):
        children = list()
        path = node_path(self.model)
        if path and len(path) > 1 and path[0] == node.name:
            curpath = path[1]
        else:
            curpath = ''
        for key in node.keys():
            child = node[key]
            props = child.properties
            if self.ignore_node(child, props):
                continue
            selected = curpath == key
            item = self.create_item(child, props, False, selected)
            children.append(item)
        return children

    def ignore_node(self, node, props):
        if props.skip_mainmenu:
            return True
        if not self.request.has_permission('view', node):
            return True
        return False

    def create_item(self, node, props, empty_title, selected):
        md = node.metadata
        item = dict()
        item['id'] = node.name
        if empty_title:
            item['title'] = ''
            item['description'] = md.title
        else:
            item['title'] = md.title
            item['description'] = md.description
        item['url'] = make_url(self.request, node=node)
        query = make_query(contenttile=props.default_content_tile)
        item['target'] = make_url(self.request, node=node, query=query)
        item['selected'] = selected
        item['icon'] = node_icon(node)
        return item


@tile(name='pathbar',
      path='templates/pathbar.pt',
      permission='view',
      strict=False)
class PathBar(Tile):

    @property
    def items(self):
        return self.items_for(self.model)

    def item_url(self, node):
        return make_url(self.request, node=node)

    def item_target(self, node):
        query = make_query(contenttile=node.properties.default_content_tile)
        return make_url(self.request, node=node, query=query)

    def items_for(self, model, breakpoint=None):
        items = list()
        for node in LocationIterator(model):
            title = node.metadata.title
            title = safe_decode(title) if title else title
            items.append({
                'title': title,
                'url': self.item_url(node),
                'target': self.item_target(node),
                'selected': False,
                'id': node.name,
                'default_child': node.properties.default_child,
            })
            if node is breakpoint:
                break
        items.reverse()
        ret = list()
        count = len(items)
        for i in range(count):
            default_child = items[i]['default_child']
            if default_child \
                    and i < count - 1 \
                    and default_child == items[i + 1]['id']:
                continue
            ret.append(items[i])

        # XXX: this is crap!
        if not breakpoint:
            ret[0]['title'] = 'Home'

        ret[-1]['selected'] = True
        return ret


@tile(name='navtree',
      path='templates/navtree.pt',
      permission='view',
      strict=False)
class NavTree(Tile):
    """Navigation tree tile.
    """

    @property
    def title(self):
        navroot = self.navroot
        default = _('navigation', default='Navigation')
        if self.model.root is navroot:
            return default
        return navroot.metadata.get('title', default)

    @property
    def navroot(self):
        model = self.model
        root = model.root
        while model is not root:
            if model.properties.is_navroot:
                return model
            model = model.parent
        return root

    def navtreeitem(self, title, url, target, path, icon, css=''):
        item = dict()
        item['title'] = title
        item['url'] = url
        item['target'] = target
        item['selected'] = False
        item['path'] = path
        item['icon'] = icon
        item['css'] = css
        item['showchildren'] = False
        item['children'] = list()
        return item

    def fillchildren(self, model, path, tree):
        """XXX: consider cone.app.interfaces.INavigationLeaf
        """
        curpath = None
        if path:
            curpath = path[0]
        default_child = None
        if model.properties.default_child:
            if not curpath or curpath == model.properties.default_child:
                default_child = model[model.properties.default_child]
        if default_child and default_child.properties.hide_if_default:
            model = default_child
            default_child = None
            if path:
                path = path[1:]
                if path:
                    curpath = path[0]
        if default_child:
            if not curpath:
                curpath = model.properties.default_child
        for key in model:
            node = model[key]
            if not self.request.has_permission('view', node):
                continue
            if not node.properties.get('in_navtree'):
                continue
            title = node.metadata.title
            if title:
                title = safe_decode(title)
            url = make_url(self.request, node=node)
            query = make_query(contenttile=node.properties.default_content_tile)
            target = make_url(self.request, node=node, query=query)
            curnode = curpath == safe_decode(key)
            icon = node_icon(node)
            css = ''
            if IWorkflowState.providedBy(node):
                css = 'state-%s' % node.state
            child = self.navtreeitem(
                title, url, target, node_path(node), icon, css)
            child['showchildren'] = curnode
            if curnode:
                child['selected'] = True
                if default_child:
                    self.fillchildren(default_child, path[1:], child)
                else:
                    self.fillchildren(node, path[1:], child)
            else:
                selected_path = node_path(self.model)
                if default_child:
                    selected_path.append(default_child.name)
                selected = False
                # XXX: probably superfluous. keep as of cone.app 1.1
                # if selected_path == node_path(node):
                #     selected = True
                child['selected'] = selected
            tree['children'].append(child)

    def navtree(self):
        root = self.navtreeitem(None, None, None, '', None)
        model = self.navroot
        # XXX: default child
        path = node_path(self.model)[len(node_path(model)):]
        self.fillchildren(model, path, root)
        return root

    def rendertree(self, children, level=1):
        return render_template(
            'cone.app.browser:templates/navtree_recue.pt',
            model=self.model,
            request=self.request,
            context=self,
            children=children,
            level=level)


@tile(name='byline',
      path='templates/byline.pt',
      permission='view',
      strict=False)
class Byline(Tile):
    """Byline tile.
    """

    def format_date(self, dt):
        return format_date(dt)


@tile(name='content', interface=AppRoot, permission='login')
class RootContent(ProtectedContentTile):

    def render(self):
        default_child = self.model.properties.default_child
        if default_child:
            model = self.model[default_child]
            return render_tile(model, self.request, 'content')
        default_content_tile = self.model.properties.default_content_tile
        if default_content_tile:
            return render_tile(self.model, self.request, default_content_tile)
        return render_template(
            'cone.app.browser:templates/default_root.pt',
            model=self.model,
            request=self.request,
            context=self)



from cone.app import testing
from cone.app.browser.actions import LinkAction
from cone.app.browser.referencebrowser import ActionAddReference
from cone.app.browser.referencebrowser import ActionRemoveReference
from cone.app.browser.referencebrowser import ReferencableChildrenLink
from cone.app.browser.referencebrowser import ReferenceAction
from cone.app.browser.referencebrowser import ReferenceBrowserModelMixin
from cone.app.browser.referencebrowser import wrap_ajax_target
from cone.app.interfaces import INavigationLeaf
from cone.app.model import BaseNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from datetime import datetime
from datetime import timedelta
from node.behaviors import UUIDAware
from plumber import plumbing
from pyramid.httpexceptions import HTTPForbidden
from yafowil.base import ExtractionError
from yafowil.base import factory
from yafowil.base import RuntimeData
from yafowil.base import Widget
from zope.interface import implementer


@plumbing(UUIDAware)
class RefNode(BaseNode):
    node_info_name = 'ref_node'


class TestBrowserReferenceBrowser(TileTestCase):
    layer = testing.security

    def test_single_valued(self):
        # Render without any value
        widget = factory(
            'reference',
            name='ref',
            props={
                'label': 'Reference',
                'multivalued': False,
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            })
        self.checkOutput("""
        <span
        ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        class="referencebrowser" id="input-ref" name="ref" readonly="readonly"
        type="text" value="" /><input name="ref.uid" type="hidden" value="" /></span>
        """, widget())

        # Render required with empty value
        widget = factory(
            'reference',
            name='ref',
            props={
                'label': 'Reference',
                'multivalued': False,
                'required': 'Ref Required',
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            })

        request = self.layer.new_request()
        request.params['ref'] = ''
        request.params['ref.uid'] = ''

        data = widget.extract(request)
        self.assertEqual(data.extracted, '')
        self.assertEqual(data.errors, [ExtractionError('Ref Required')])

        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        class="referencebrowser required" id="input-ref" name="ref" readonly="readonly" type="text"
        value="" /><input name="ref.uid" type="hidden" value="" /></span>
        """, widget(data=data))

        # Required with valid value
        request.params['ref'] = 'Title'
        request.params['ref.uid'] = '123'
        data = widget.extract(request)
        self.assertEqual(data.extracted, '123')
        self.assertEqual(data.errors, [])

        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        class="referencebrowser required" id="input-ref" name="ref" readonly="readonly" type="text"
        value="Title" /><input name="ref.uid" type="hidden" value="123" /></span>
        """, widget(data=data))

        # Single valued expects 2-tuple as value with (uid, label)
        widget = factory(
            'reference',
            name='ref',
            value=('uid', 'Label'),
            props={
                'label': 'Reference',
                'multivalued': False,
                'required': 'Ref Required',
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            })
        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected=uid"><input
        class="referencebrowser required" id="input-ref" name="ref" readonly="readonly" type="text"
        value="Label" /><input name="ref.uid" type="hidden" value="uid" /></span>
        """, widget())

        # Extract from request and render widget with data
        data = widget.extract(request)
        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected=uid"><input
        class="referencebrowser required" id="input-ref" name="ref" readonly="readonly" type="text"
        value="Title" /><input name="ref.uid" type="hidden" value="123" /></span>
        """, widget(data=data))

        # Render widget with request
        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected=uid"><input
        class="referencebrowser required" id="input-ref" name="ref" readonly="readonly" type="text"
        value="Title" /><input name="ref.uid" type="hidden" value="123" /></span>
        """, widget(request=request))

        # Single value display renderer
        widget = factory(
            'reference',
            name='ref',
            props={
                'label': 'Reference',
                'multivalued': False,
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            },
            mode='display')
        self.assertEqual(
            widget(),
            u'<div class="display-referencebrowser" id="display-ref"></div>'
        )

        widget = factory(
            'reference',
            name='ref',
            value=('uid', 'Label'),
            props={
                'label': 'Reference',
                'multivalued': False,
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            },
            mode='display')
        self.assertEqual(
            widget(),
            u'<div class="display-referencebrowser" id="display-ref">Label</div>'
        )

    def test_multi_valued(self):
        # Render without any value
        widget = factory(
            'reference',
            name='ref',
            props={
                'label': 'Reference',
                'multivalued': True,
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
            })
        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        id="exists-ref" name="ref-exists" type="hidden" value="exists" /><select
        class="referencebrowser" id="input-ref" multiple="multiple"
        name="ref" /></span>
        """, widget())

        # Render required with empty value
        widget = factory(
            'reference',
            name='ref',
            props={
                'label': 'Reference',
                'multivalued': True,
                'required': 'Ref Required',
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
                'vocabulary': [
                    ('uid1', 'Title1'),
                    ('uid2', 'Title2'),
                ],
            })

        request = self.layer.new_request()
        request.params['ref'] = ''

        data = widget.extract(request)
        self.assertEqual(data.extracted, '')
        self.assertEqual(data.errors, [ExtractionError('Ref Required',)])

        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        id="exists-ref" name="ref-exists" type="hidden" value="exists" /><select
        class="referencebrowser required" id="input-ref" multiple="multiple"
        name="ref" required="required"><option
        id="input-ref-uid1" value="uid1">Title1</option><option
        id="input-ref-uid2" value="uid2">Title2</option></select></span>
        """, widget(data=data))

        # Required with valid value
        request.params['ref'] = ['uid1', 'uid2']
        data = widget.extract(request)
        self.assertEqual(data.extracted, ['uid1', 'uid2'])
        self.assertEqual(data.errors, [])

        self.checkOutput("""
        <span ajax:target="http://example.com/foo?referencable=ref_node&root=/&selected="><input
        id="exists-ref" name="ref-exists" type="hidden" value="exists" /><select
        class="referencebrowser required" id="input-ref"
        multiple="multiple" name="ref" required="required"><option
        id="input-ref-uid1" selected="selected" value="uid1">Title1</option><option
        id="input-ref-uid2" selected="selected"
        value="uid2">Title2</option></select></span>
        """, widget(data=data))

        # Multi value display renderer
        widget = factory(
            'reference',
            name='ref',
            value=['uid1', 'uid2'],
            props={
                'label': 'Reference',
                'target': 'http://example.com/foo',
                'referencable': 'ref_node',
                'multivalued': True,
                'vocabulary': [
                    ('uid1', 'Title1'),
                    ('uid2', 'Title2'),
                ],
            },
            mode='display')
        self.checkOutput("""
        <ul class="display-referencebrowser"
        id="display-ref"><li>Title1</li><li>Title2</li></ul>
        """, widget())

    def test_wrap_ajax_target(self):
        widget = Widget(
            blueprints='dummy',
            extractors=[],
            edit_renderers=[],
            display_renderers=[],
            preprocessors=[],
            uniquename='dummy')
        data = RuntimeData()

        # XXX: referencebrowser not works without target, check if case
        #      necessary
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, 'rendered')

        widget.attrs['multivalued'] = False
        widget.attrs['target'] = 'http://example.com'
        widget.attrs['referencable'] = 'ref_node'
        widget.attrs['root'] = '/'
        expected = (
            '<span ajax:target='
            '"http://example.com?referencable=ref_node&root=/&selected="'
            '>rendered</span>'
        )
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

        def callable_target(widget, data):
            return 'http://example.com'
        widget.attrs['target'] = callable_target

        def callable_referencable(widget, data):
            return 'ref_node'
        widget.attrs['referencable'] = callable_referencable

        def callable_root(widget, data):
            return '/'
        widget.attrs['root'] = callable_root

        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

        widget.attrs['referencable'] = ['ref_node']
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

        widget.attrs['referencable'] = ['ref_node', 'ref_node_2']
        expected = (
            '<span ajax:target='
            '"http://example.com?referencable=ref_node,ref_node_2&root=/&selected="'
            '>rendered</span>'
        )
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

        widget.attrs['referencable'] = 'ref_node'
        data.value = ('sel_ref', 'Selected Reference')
        expected = (
            '<span ajax:target='
            '"http://example.com?referencable=ref_node&root=/&selected=sel_ref"'
            '>rendered</span>'
        )
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

        widget.attrs['multivalued'] = True
        data.value = ('sel_ref', 'sel_ref_2')
        expected = (
            '<span ajax:target='
            '"http://example.com?referencable=ref_node&root=/&selected=sel_ref,sel_ref_2"'
            '>rendered</span>'
        )
        rendered = wrap_ajax_target('rendered', widget, data)
        self.assertEqual(rendered, expected)

    def test_ReferenceBrowserModelMixin(self):
        model = BaseNode()
        a = model['a'] = BaseNode()
        aa = model['a']['a'] = BaseNode()
        b = model['b'] = BaseNode()

        ref_model_mixin = ReferenceBrowserModelMixin()
        ref_model_mixin.model = model

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/'
        self.assertEqual(ref_model_mixin.referencable_root, model)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencable_root, model)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/a'
        self.assertEqual(ref_model_mixin.referencable_root, a)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/a/a'
        self.assertEqual(ref_model_mixin.referencable_root, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'b'
        self.assertEqual(ref_model_mixin.referencable_root, b)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'c'
        self.expectError(
            KeyError,
            lambda: ref_model_mixin.referencable_root
        )

        ref_model_mixin.model = aa

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a/a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        ref_model_mixin.model = b

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencebrowser_model, b)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, a)

    def test_ReferenceAction(self):
        model = RefNode()
        request = self.layer.new_request()

        action = ReferenceAction()
        action.model = model
        action.request = request

        request.params['selected'] = ''
        self.assertEqual(action.selected_uids, [])
        request.params['selected'] = 'foo'
        self.assertEqual(action.selected_uids, ['foo'])
        request.params['selected'] = 'foo,bar'
        self.assertEqual(action.selected_uids, ['foo', 'bar'])

        request.params['referencable'] = 'ref_node,ref_node_2'
        self.assertTrue(action.display)
        request.params['referencable'] = 'ref_node'
        self.assertTrue(action.display)
        request.params['referencable'] = 'ref_node_2'
        self.assertFalse(action.display)
        request.params['referencable'] = ''
        self.assertFalse(action.display)

        self.checkOutput('ref-...', action.id)
        self.checkOutput("""
        ...<a
        id="ref-..."
        href="http://example.com/"
        data-toggle="tooltip"
        data-placement="top"
        ajax:bind="click"
        ></a>...
        """, action.render())

        model = BaseNode()
        model.node_info_name = 'nouuid'
        request.params['referencable'] = 'nouuid'

        action.model = model
        self.assertFalse(action.display)

    def test_ActionAddReference(self):
        model = BaseNode()
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        action = ActionAddReference()
        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            self.assertEqual(action(model, request), u'')

        model = RefNode(name='model')
        rendered = action(model, request)
        expected = 'class="addreference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = 'title="Add reference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="ion-plus-round">'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="reftitle" style="display:none;">model</span>'
        self.assertTrue(rendered.find(expected) > -1)

        request.params['selected'] = str(model.uuid)
        rendered = action(model, request)
        expected = 'class="addreference disabled"'
        self.assertTrue(rendered.find(expected) > -1)

        action = ActionAddReference()
        action.model = BaseNode(name='model')
        action.request = self.layer.new_request()
        self.assertFalse(action.enabled)

    def test_ActionRemoveReference(self):
        model = BaseNode()
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        action = ActionRemoveReference()
        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            self.assertEqual(action(model, request), u'')

        model = RefNode(name='model')
        rendered = action(model, request)
        expected = 'class="removereference disabled"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = 'title="Remove reference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="ion-minus-round">'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="reftitle" style="display:none;">model</span>'
        self.assertTrue(rendered.find(expected) > -1)

        request.params['selected'] = str(model.uuid)
        rendered = action(model, request)
        expected = 'class="removereference"'
        self.assertTrue(rendered.find(expected) > -1)

        action = ActionRemoveReference()
        action.model = BaseNode(name='model')
        action.request = self.layer.new_request()
        self.assertFalse(action.enabled)

    def test_ReferencableChildrenLink(self):
        model = BaseNode(name='model')
        model.metadata.title = 'My Node'
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        table_tile_name = 'tabletile'
        table_id = 'tableid'

        action = ReferencableChildrenLink(table_tile_name, table_id)
        action.model = model
        action.request = request

        self.assertTrue(isinstance(action, LinkAction))

        expected = 'http://example.com/model?referencable=ref_node&root=/&selected='
        self.assertEqual(action.target, expected)

        expected = 'My Node'
        self.assertEqual(action.text, expected)

        expected = 'tabletile:#tableid:replace'
        self.assertEqual(action.action, expected)

        expected = 'contextchanged:.refbrowsersensitiv'
        self.assertEqual(action.event, expected)

        self.assertFalse(action.display)
        with self.layer.authenticated('max'):
            self.assertTrue(action.display)

        expected = 'glyphicon glyphicon-asterisk'
        self.assertEqual(action.icon, expected)

        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            rendered = action(model, request)
        expected = '<a'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '&nbsp;My Node</a>'
        self.assertTrue(rendered.find(expected) > -1)

        @implementer(INavigationLeaf)
        class LeafNode(BaseNode):
            pass

        model = LeafNode(name='leaf')
        with self.layer.authenticated('manager'):
            rendered = action(model, request)
        expected = '<span class="glyphicon glyphicon-asterisk" />&nbsp;<span>leaf</span>'
        self.assertEqual(rendered, expected)

    def test_reference_pathbar(self):
        model = BaseNode()
        model['a'] = RefNode()
        model['a']['b'] = RefNode()
        model['z'] = RefNode()
        node = model['a']['b']['c'] = RefNode()

        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        # Case Unauthorized
        err = self.expectError(
            HTTPForbidden,
            render_tile,
            node,
            request,
            'referencebrowser_pathbar'
        )
        self.checkOutput("""
        Unauthorized: tile
        <cone.app.browser.referencebrowser.ReferenceBrowserPathBar object at ...>
        failed permission check
        """, str(err))

        # Case reference root is application root
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertTrue(res.find('"http://example.com/?') > -1)
        self.assertTrue(res.find('"http://example.com/a?') > -1)
        self.assertTrue(res.find('"http://example.com/a/b?') > -1)

        # Case reference root is in current sub tree
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = 'a'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertFalse(res.find('"http://example.com/?') > -1)
        self.assertTrue(res.find('"http://example.com/a?') > -1)
        self.assertTrue(res.find('"http://example.com/a/b?') > -1)

        # Case reference root is in sibling sub tree
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/z'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertFalse(res.find('"http://example.com/?') > -1)
        self.assertFalse(res.find('"http://example.com/a?') > -1)
        self.assertFalse(res.find('"http://example.com/a/b?') > -1)
        self.assertTrue(res.find('<strong>z</strong>') > -1)

    def test_reference_listing(self):
        created = datetime(2011, 3, 15)
        delta = timedelta(1)
        modified = created + delta

        model = BaseNode()
        for i in range(20):
            model[str(i)] = RefNode()
            # set listing display metadata
            model[str(i)].metadata.title = str(i)
            model[str(i)].metadata.created = created
            model[str(i)].metadata.modified = modified
            if i % 2 == 0:
                # make node referencable
                model[str(i)].properties.action_add_reference = True
                # do not render link to children
                model[str(i)].properties.leaf = True
            created = created + delta
            modified = modified + delta

        # Unauthorized fails
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        err = self.expectError(
            HTTPForbidden,
            render_tile,
            model,
            request,
            'referencelisting'
        )
        self.checkOutput("""
            Unauthorized: tile
            <cone.app.browser.referencebrowser.ReferenceListing object at ...>
            failed permission check
        """, str(err))

        # Authorized
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'referencelisting')
        self.assertTrue(res.find('id="referencebrowser"') > -1)
        self.checkOutput('...<div id="referencebrowser"...', res)

        # Referencable nodes renders add reference action related markup
        self.checkOutput("""
        ...
        <a
        id="ref-..."
        href="http://example.com/1"
        class="addreference"
        title="Add reference"
        data-toggle="tooltip"
        data-placement="top"
        ajax:bind="click"
        ><span class="ion-plus-round"></span></a>...
        """, res)

    def test_referencebrowser(self):
        model = BaseNode()
        model['a'] = RefNode()

        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        # Case unauthorized
        err = self.expectError(
            HTTPForbidden,
            render_tile,
            model,
            request,
            'referencebrowser'
        )
        self.checkOutput("""
            Unauthorized: tile
            <cone.app.browser.referencebrowser.ReferenceBrowser object at ...>
            failed permission check
        """, str(err))

        # Case authorized
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'referencebrowser')
        self.assertTrue(res.find('<div class="referencebrowser">') > -1)
        self.assertTrue(res.find('<div id="referencebrowser_pathbar"') > -1)
        self.assertTrue(res.find('<div id="referencebrowser"') > -1)



from cone.app import testing
from cone.app.browser.actions import Action
from cone.app.browser.actions import ActionContext
from cone.app.browser.actions import LinkAction
from cone.app.browser.contextmenu import ContextMenuDropdown
from cone.app.browser.contextmenu import ContextMenuToolbar
from cone.app.model import BaseNode
from cone.app.testing.mock import CopySupportNode
from cone.app.testing.mock import SharingNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase


class TestBrowserContextmenu(TileTestCase):
    layer = testing.security

    def test_ContextMenuToolbar(self):
        model = BaseNode()
        request = self.layer.new_request()

        cmt = ContextMenuToolbar()
        cmt.display = False
        self.assertEqual(cmt(model, request), '')

        class EmptyRenderAction(Action):
            def render(self):
                return ''

        cmt = ContextMenuToolbar()
        cmt['empty'] = EmptyRenderAction()
        self.assertEqual(cmt(model, request), '')

        class RenderAction(Action):
            def render(self):
                return '<span>Rendered action</span>'

        cmt = ContextMenuToolbar()
        cmt['action'] = RenderAction()
        self.assertEqual(
            cmt(model, request),
            '<ul class="nav navbar-nav"><span>Rendered action</span></ul>'
        )

        class MyLinkAction(LinkAction):
            id = 'myaction'
            icon = 'myicon'
            text = 'My Action'
            href = 'http://example.com'

        cmt = ContextMenuToolbar()
        cmt['link'] = MyLinkAction()
        self.assertTrue(cmt(model, request).find('My Action') > -1)
        self.assertFalse(cmt(model, request).find('<li class="active">') > -1)

        cmt['link'].selected = True
        self.assertTrue(cmt(model, request).find('<li class="active">') > -1)

        cmt.css = 'mytoolbar'
        expected = 'class="nav navbar-nav mytoolbar"'
        self.assertTrue(cmt(model, request).find(expected) > -1)

    def test_ContextMenuDropdown(self):
        cmd = ContextMenuDropdown(title='Dropdown')

        def add_action():
            cmd['invalid'] = Action()
        err = self.expectError(ValueError, add_action)
        expected = (
            'Only ``LinkAction`` deriving objects can be added '
            'to ``ContextMenuDropdown`` instances.'
        )
        self.assertEqual(str(err), expected)

        class MyLinkAction(LinkAction):
            id = 'myaction'
            icon = 'myicon'
            text = 'My Action'
            href = 'http://example.com'

        cmd = ContextMenuDropdown(title='Dropdown')
        cmd.model = BaseNode()
        cmd.request = self.layer.new_request()

        cmd['link'] = MyLinkAction()
        self.assertTrue(cmd.display)

        cmd['link'].display = False
        self.assertFalse(cmd.display)

        cmd = ContextMenuDropdown(title='Dropdown')
        cmd['link'] = MyLinkAction()

        model = BaseNode()
        request = self.layer.new_request()
        res = cmd(model, request)
        self.assertTrue(res.find('<li class="dropdown">') > -1)

    def test_contextmenu(self):
        with self.layer.authenticated('manager'):
            parent = BaseNode('root')
            model = parent['model'] = SharingNode()
            model.properties.action_up = True
            model.properties.action_view = True
            model.properties.action_list = True
            model.properties.action_edit = True
            model.properties.action_delete = True
            # XXX: model.properties.action_delete_children = True

            request = self.layer.new_request()
            ActionContext(model, request, 'content')
            rendered = render_tile(model, request, 'contextmenu')

            self.assertTrue(rendered.find('toolbaraction-up') > -1)
            self.assertTrue(rendered.find('toolbaraction-view') > -1)
            self.assertTrue(rendered.find('toolbaraction-list') > -1)
            self.assertTrue(rendered.find('toolbaraction-edit') > -1)
            self.assertTrue(rendered.find('toolbaraction-delete') > -1)
            self.assertTrue(rendered.find('toolbaraction-share') > -1)

            model = CopySupportNode()
            ActionContext(model, request, 'listing')
            rendered = render_tile(model, request, 'contextmenu')

            self.assertTrue(rendered.find('toolbaraction-cut') > -1)
            self.assertTrue(rendered.find('toolbaraction-copy') > -1)
            self.assertTrue(rendered.find('toolbaraction-paste') > -1)



from cone.app import compat
from cone.app import testing
from cone.app.browser import set_related_view
from cone.app.browser.contents import ContentsTile
from cone.app.browser.contents import ContentsViewLink
from cone.app.browser.contents import listing
from cone.app.browser.table import TableSlice
from cone.app.browser.utils import make_url
from cone.app.model import BaseNode
from cone.app.model import node_info
from cone.app.testing.mock import CopySupportNode
from cone.app.testing.mock import WorkflowNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from datetime import datetime
from datetime import timedelta
from pyramid.exceptions import HTTPForbidden
from pyramid.security import ACLDenied
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Deny
from pyramid.security import Everyone


class NeverShownChild(BaseNode):
    __acl__ = [(Deny, Everyone, ALL_PERMISSIONS)]


class TestBrowserContents(TileTestCase):
    layer = testing.security

    def create_dummy_model(self):
        created = datetime(2011, 3, 14)
        delta = timedelta(1)
        modified = created + delta
        model = BaseNode()
        for i in range(19):
            model[str(i)] = BaseNode()
            model[str(i)].properties.action_view = True
            model[str(i)].properties.action_edit = True
            model[str(i)].properties.action_delete = True
            model[str(i)].metadata.title = str(i) + ' Title'
            model[str(i)].metadata.creator = 'admin ' + str(19 - i)
            model[str(i)].metadata.created = created
            model[str(i)].metadata.modified = modified
            created = created + delta
            modified = modified + delta
        model['nevershown'] = NeverShownChild()
        return model

    def test_sorted_rows(self):
        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = self.create_dummy_model()
        contents.request = self.layer.new_request()

        # ``sorted_rows`` returns sorted listing items. ``start``, ``end``,
        # ``sort`` and ``order`` are expected by this function
        with self.layer.authenticated('manager'):
            res = contents.sorted_rows(None, None, 'created', 'desc')[0]['title']
            self.checkOutput('...0 Title...', res)

            res = contents.sorted_rows(None, None, 'created', 'desc')[-1]['title']
            self.checkOutput('...18 Title...', res)

            res = contents.sorted_rows(None, None, 'created', 'asc')[0]['title']
            self.checkOutput('...18 Title...', res)

            res = contents.sorted_rows(None, None, 'created', 'asc')[-1]['title']
            self.checkOutput('...0 Title...', res)

    def test_slice(self):
        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = self.create_dummy_model()
        request = contents.request = self.layer.new_request()

        # ``contents.slice.slice`` return current batch start and positions
        self.assertTrue(isinstance(contents.slice, TableSlice))
        self.assertEqual(contents.slice.slice, (0, 15))

        request.params['b_page'] = '1'
        self.assertEqual(contents.slice.slice, (15, 30))

        del request.params['b_page']

        # ``contents.slice.rows`` return the current sorted row data for listing.
        with self.layer.authenticated('manager'):
            # Items returned by default sorting
            res = contents.slice.rows[0]['title']
            self.checkOutput('...0 Title...', res)

            res = contents.slice.rows[-1]['title']
            self.checkOutput('...14 Title...', res)

            # Inverse order
            request.params['order'] = 'asc'
            res = contents.slice.rows[0]['title']
            self.checkOutput('...18 Title...', res)

            res = contents.slice.rows[-1]['title']
            self.checkOutput('...4 Title...', res)

            # Switch batch page with inversed order
            request.params['b_page'] = '1'
            res = contents.slice.rows[0]['title']
            self.checkOutput('...3 Title...', res)

            res = contents.slice.rows[-1]['title']
            self.checkOutput('...0 Title...', res)

            # Reset order and batch page
            del request.params['order']
            del request.params['b_page']

            # Sort by creator
            request.params['sort'] = 'creator'
            self.assertEqual([row['creator'] for row in contents.slice.rows], [
                'admin 1', 'admin 10', 'admin 11', 'admin 12', 'admin 13',
                'admin 14', 'admin 15', 'admin 16', 'admin 17', 'admin 18',
                'admin 19', 'admin 2', 'admin 3', 'admin 4', 'admin 5'
            ])

            request.params['b_page'] = '1'
            self.assertEqual([row['creator'] for row in contents.slice.rows], [
                'admin 6', 'admin 7', 'admin 8', 'admin 9'
            ])

            # Sort by created
            request.params['b_page'] = '0'
            request.params['sort'] = 'created'

            self.assertEqual(
                contents.slice.rows[0]['created'],
                datetime(2011, 3, 14, 0, 0)
            )
            self.assertEqual(
                contents.slice.rows[-1]['created'],
                datetime(2011, 3, 28, 0, 0)
            )

            request.params['b_page'] = '1'
            request.params['sort'] = 'modified'

            self.assertEqual(
                contents.slice.rows[0]['modified'],
                datetime(2011, 3, 30, 0, 0)
            )

            self.assertEqual(
                contents.slice.rows[-1]['modified'],
                datetime(2011, 4, 2, 0, 0)
            )

            del request.params['b_page']
            del request.params['sort']

    def test_batch(self):
        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = self.create_dummy_model()
        request = contents.request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            rendered = contents.batch

        expected = (
            '<li class="active">\n          '
            '<a href="javascript:void(0)">1</a>'
        )
        self.assertTrue(rendered.find(expected) != -1)

        expected = (
            'http://example.com/?b_page=1&amp;'
            'order=desc&amp;size=15&amp;sort=created'
        )
        self.assertTrue(rendered.find(expected) != -1)

        # Change page
        request.params['b_page'] = '1'
        with self.layer.authenticated('manager'):
            rendered = contents.batch

        expected = (
            '<li class="active">\n          '
            '<a href="javascript:void(0)">2</a>'
        )
        self.assertTrue(rendered.find(expected) != -1)

        expected = (
            'http://example.com/?b_page=0&amp;'
            'order=desc&amp;size=15&amp;sort=created'
        )
        self.assertTrue(rendered.find(expected) != -1)

        # Change sort and order. Sort is proxied by batch
        request.params['sort'] = 'modified'
        with self.layer.authenticated('manager'):
            rendered = contents.batch

        expected = (
            'http://example.com/?b_page=0&amp;'
            'order=desc&amp;size=15&amp;sort=modified'
        )
        self.assertTrue(rendered.find(expected) != -1)

    def test_authenticated(self):
        model = self.create_dummy_model()
        request = self.layer.new_request()

        # Rendering fails unauthorized, 'list' permission is required
        rule = request.has_permission('list', model)
        self.assertTrue(isinstance(rule, ACLDenied))

        err = self.expectError(
            HTTPForbidden,
            render_tile,
            model,
            request,
            'contents'
        )
        self.checkOutput("""
        Unauthorized: tile
        <cone.app.browser.contents.ContentsTile object at ...> failed
        permission check
        """, str(err))

        # Render authenticated
        with self.layer.authenticated('manager'):
            request.params['sort'] = 'modified'
            request.params['b_page'] = '1'
            rendered = render_tile(model, request, 'contents')
            expected = (
                '<a href="http://example.com/?b_page=1&amp;'
                'order=desc&amp;size=15&amp;sort=title"'
            )
            self.assertTrue(rendered.find(expected) != -1)

    def test_copysupport(self):
        # Copysupport Attributes
        model = CopySupportNode()
        model['child'] = CopySupportNode()
        request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            rendered = render_tile(model, request, 'contents')

        expected = 'class="selectable copysupportitem"'
        self.assertTrue(rendered.find(expected) > -1)

        with self.layer.authenticated('manager'):
            request = self.layer.new_request()
            cut_url = compat.quote(make_url(request, node=model['child']))
            request.cookies['cone.app.copysupport.cut'] = cut_url
            rendered = render_tile(model, request, 'contents')

        expected = 'class="selectable copysupportitem copysupport_cut"'
        self.assertTrue(rendered.find(expected) > -1)

    def test_ContentsViewLink(self):
        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')

        model = BaseNode(name='root')
        request = self.layer.new_request()

        view_link = contents.view_link
        self.assertTrue(isinstance(contents.view_link, ContentsViewLink))

        with self.layer.authenticated('max'):
            res = view_link(model, request)
        expected = 'ajax:target="http://example.com/root"'
        self.assertTrue(res.find(expected) > -1)

        set_related_view(request, 'someview')
        with self.layer.authenticated('max'):
            res = view_link(model, request)
        expected = 'ajax:target="http://example.com/root?contenttile=someview"'
        self.assertTrue(res.find(expected) > -1)

    def test_workflow_state(self):
        root = BaseNode(name='root')
        root['wf'] = WorkflowNode()
        request = self.layer.new_request()

        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = root
        contents.request = request

        with self.layer.authenticated('max'):
            res = contents.sorted_rows(
                0,
                1,
                contents.default_sort,
                contents.default_order
            )
        self.assertEqual(res[0].css, ' state-initial')

    @testing.reset_node_info_registry
    def test_node_type(self):
        @node_info(
            name='mynode')
        class MyNode(BaseNode):
            pass

        root = BaseNode()
        root['child'] = MyNode()
        request = self.layer.new_request()

        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = root
        contents.request = request

        with self.layer.authenticated('max'):
            res = contents.sorted_rows(
                0,
                1,
                contents.default_sort,
                contents.default_order
            )
        self.assertEqual(res[0].css, ' node-type-mynode')

    def test_filtered_children(self):
        root = BaseNode()

        child_1 = root['1'] = BaseNode()
        md_1 = child_1.metadata
        md_1.title = 'Child 1'
        md_1.creator = 'max'

        child_2 = root['2'] = BaseNode()
        md_2 = child_2.metadata
        md_2.title = 'Child 2'
        md_2.creator = 'sepp'

        tmpl = 'cone.app:browser/templates/table.pt'
        contents = ContentsTile(tmpl, None, 'contents')
        contents.model = root
        contents.request = self.layer.new_request()

        res = contents.filtered_children
        self.assertEqual(res, [])

        contents.request = self.layer.new_request()
        with self.layer.authenticated('max'):
            res = contents.filtered_children
        self.assertEqual(len(res), 2)

        request = contents.request = self.layer.new_request()
        request.params['term'] = 'sepp'
        with self.layer.authenticated('max'):
            res = contents.filtered_children
        self.assertEqual(len(res), 1)

        request = contents.request = self.layer.new_request()
        request.params['term'] = 'Child 1'
        with self.layer.authenticated('max'):
            res = contents.filtered_children
        self.assertEqual(len(res), 1)

        request = contents.request = self.layer.new_request()
        request.params['term'] = 'hild'
        with self.layer.authenticated('max'):
            res = contents.filtered_children
        self.assertEqual(len(res), 2)

        request = contents.request = self.layer.new_request()
        request.params['term'] = 'foo'
        with self.layer.authenticated('max'):
            res = contents.filtered_children
        self.assertEqual(len(res), 0)

    def test_listing(self):
        model = self.create_dummy_model()
        request = self.layer.new_request()

        with self.layer.authenticated('max'):
            res = listing(model, request)
        self.assertTrue(res.text.startswith('<!DOCTYPE html>'))



from cone.app.model import AppNode
from cone.app.model import BaseNode
from cone.app.model import CopySupport
from cone.app.model import Properties
from cone.app.security import DEFAULT_ACL
from cone.app.security import PrincipalACL
from cone.app.workflow import WorkflowACL
from cone.app.workflow import WorkflowState
from node.behaviors import Adopt
from node.behaviors import Attributes
from node.behaviors import DefaultInit
from node.behaviors import Nodespaces
from node.behaviors import Nodify
from node.behaviors import OdictStorage
from node.utils import instance_property
from plumber import plumbing
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow
from pyramid.security import Deny
from pyramid.security import Everyone
from pyramid.static import static_view


static_resources = static_view('static', use_subpath=True)


@plumbing(WorkflowState, WorkflowACL)
class WorkflowNode(BaseNode):
    workflow_name = u'dummy'
    workflow_tsf = None

    @property
    def properties(self):
        props = Properties()
        props.in_navtree = True
        return props

    def __call__(self):
        pass


class InexistentWorkflowNode(WorkflowNode):
    workflow_name = u'inexistent'
    workflow_tsf = None


class StateACLWorkflowNode(WorkflowNode):
    state_acls = {
        'initial': [
            (Allow, 'role:manager', ['manage', 'edit', 'change_state']),
            (Allow, Everyone, ['login']),
            (Deny, Everyone, ALL_PERMISSIONS),
        ],
        'final': [
            (Allow, 'role:manager', ['view', 'edit', 'change_state']),
            (Deny, Everyone, ALL_PERMISSIONS),
        ],
    }


@plumbing(
    PrincipalACL,
    AppNode,
    Adopt,
    Nodespaces,
    Attributes,
    DefaultInit,
    Nodify,
    OdictStorage)
class SharingNode(object):

    @property
    def __acl__(self):
        return DEFAULT_ACL

    @instance_property
    def principal_roles(self):
        return dict()


@plumbing(CopySupport)
class CopySupportNode(BaseNode):

    def __init__(self, name=None, parent=None):
        super(CopySupportNode, self).__init__(name=name, parent=parent)
        self.messages = []

    def __call__(self):
        self.messages.append('Called: {}'.format(self.name))



from cone.app import main_hook
from cone.app import make_remote_addr_middleware
from cone.app.model import BaseNode
from node.tests import NodeTestCase
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.router import Router
from pyramid.static import static_view
from yafowil import resources
from yafowil.base import factory as yafowil_factory
import cone.app


class TestApp(NodeTestCase):

    def test_get_root(self):
        root = cone.app.get_root()
        self.assertTrue(str(root).startswith("<AppRoot object 'None' at"))

        # AppRoot contains a settings node by default
        self.assertTrue('settings' in root.factories.keys())

        # Settings contains metadata.title by default
        self.assertEqual(list(root['settings'].metadata.keys()), ['title'])
        self.assertEqual(root['settings'].metadata.title, u'settings')

        # Settings is displayed in navtree by default
        self.assertEqual(
            sorted(root['settings'].properties.keys()),
            ['icon', 'in_navtree', 'skip_mainmenu']
        )
        self.assertFalse(root['settings'].properties.in_navtree)
        self.assertTrue(root['settings'].properties.skip_mainmenu)

    def test_register_plugin(self):
        cone.app.register_plugin('dummy', BaseNode)

        root = cone.app.get_root()
        self.assertTrue('dummy' in root.factories.keys())

        err = self.expect_error(
            ValueError,
            lambda: cone.app.register_plugin('dummy', BaseNode)
        )
        expected = "Entry with name 'dummy' already registered."
        self.assertEqual(str(err), expected)

    def test_register_plugin_config(self):
        cone.app.register_plugin_config('dummy', BaseNode)

        root = cone.app.get_root()
        self.assertTrue('dummy' in root['settings'].factories.keys())

        err = self.expect_error(
            ValueError,
            lambda: cone.app.register_plugin_config('dummy', BaseNode)
        )
        expected = "Config with name 'dummy' already registered."
        self.assertEqual(str(err), expected)

    def test_main(self):
        # main hook
        hooks = dict(called=0)

        def custom_main_hook(configurator, global_config, settings):
            hooks['called'] += 1

        cone.app.register_main_hook(custom_main_hook)

        @main_hook
        def decorated_main_hook(configurator, global_config, settings):
            hooks['called'] += 1

        # set auth tkt factory``
        factory = cone.app.auth_tkt_factory(secret='12345')
        self.assertTrue(isinstance(factory, AuthTktAuthenticationPolicy))

        # ACL factory
        factory = cone.app.acl_factory()
        self.assertTrue(isinstance(factory, ACLAuthorizationPolicy))

        # yafowil resources
        def dummy_get_plugin_names(ns=None):
            return ['yafowil.addon']

        get_plugin_names_origin = resources.get_plugin_names
        resources.get_plugin_names = dummy_get_plugin_names

        yafowil_addon_name = 'yafowil.addon'
        js = [{
            'group': 'yafowil.addon.common',
            'resource': 'widget.js',
            'order': 20,
        }]
        css = [{
            'group': 'yafowil.addon.common',
            'resource': 'widget.css',
            'order': 20,
        }]
        yafowil_factory.register_theme(
            'default',
            yafowil_addon_name,
            'yafowil_addon_resources',
            js=js,
            css=css
        )

        # settings
        settings = {
            'cone.admin_user': 'admin',
            'cone.admin_password': 'admin',
            'cone.auth_secret': '12345',
            'cone.auth_reissue_time': '300',
            'cone.auth_max_age': '600',
            'cone.plugins': 'cone.app.tests'  # ensure dummy main hooks called
        }

        # main
        router = cone.app.main({}, **settings)
        self.assertTrue(isinstance(router, Router))
        self.assertEqual(hooks['called'], 2)

        # Remove custom main hook after testing
        cone.app.main_hooks.remove(custom_main_hook)
        cone.app.main_hooks.remove(decorated_main_hook)

        # Check created yafowil addon static view
        self.assertTrue(isinstance(
            cone.app.yafowil_addon_resources,
            static_view
        ))

        # Remove dummy yafowil theme, reset get_plugin_names patch
        # and delete created yafowil addon static view
        resources.get_plugin_names = get_plugin_names_origin
        del yafowil_factory._themes['default'][yafowil_addon_name]
        del cone.app.yafowil_addon_resources

    def test_remote_addr_middleware(self):
        # Remote address middleware
        class DummyApp(object):
            remote_addr = None

            def __call__(self, environ, start_response):
                self.remote_addr = environ['REMOTE_ADDR']

        app = DummyApp()
        middleware = make_remote_addr_middleware(app, {})

        environ = {}
        environ['HTTP_X_REAL_IP'] = '1.2.3.4'
        middleware(environ, None)
        self.assertEqual(app.remote_addr, '1.2.3.4')



from cone.app import compat
from cone.app.browser.actions import LinkAction
from cone.app.browser.actions import Toolbar
from cone.app.browser.contents import ContentsTile
from cone.app.browser.layout import PathBar
from cone.app.browser.table import RowData
from cone.app.browser.utils import make_query
from cone.app.browser.utils import request_property
from cone.app.browser.utils import make_url
from cone.app.interfaces import INavigationLeaf
from cone.tile import Tile
from cone.tile import render_tile
from cone.tile import tile
from node.interfaces import IUUIDAware
from node.utils import LocationIterator
from node.utils import instance_property
from node.utils import node_by_path
from pyramid.i18n import TranslationStringFactory
from yafowil.base import UNSET
from yafowil.base import factory
from yafowil.base import fetch_value
from yafowil.common import generic_extractor
from yafowil.common import generic_required_extractor
from yafowil.common import select_display_renderer
from yafowil.common import select_edit_renderer
from yafowil.common import select_extractor
from yafowil.utils import Tag
from yafowil.utils import cssclasses
from yafowil.utils import cssid
import types


_ = TranslationStringFactory('cone.app')


# XXX: define i18n translation callback for app
tag = Tag(lambda x: x)


def make_refbrowser_query(request, **kw):
    params = {
        'root': request.params['root'],
        'referencable': request.params['referencable'],
        'selected': request.params['selected'],
    }
    params.update(kw)
    return make_query(**params)


class ReferenceBrowserModelMixin(object):

    @request_property
    def referencable_root(self):
        return node_by_path(
            self.model.root,
            self.request.params['root'].strip('/')
        )

    @request_property
    def referencebrowser_model(self):
        root = self.referencable_root
        if root in LocationIterator(self.model):
            return self.model
        return root


@tile(name='referencebrowser',
      path='templates/referencebrowser.pt',
      permission='view')
class ReferenceBrowser(Tile, ReferenceBrowserModelMixin):

    @property
    def referencebrowser_pathbar(self):
        return render_tile(
            self.referencebrowser_model,
            self.request,
            'referencebrowser_pathbar'
        )

    @property
    def referencelisting(self):
        return render_tile(
            self.referencebrowser_model,
            self.request,
            'referencelisting'
        )


@tile(name='referencebrowser_pathbar',
      path='templates/referencebrowser_pathbar.pt',
      permission='view')
class ReferenceBrowserPathBar(PathBar, ReferenceBrowserModelMixin):

    @property
    def items(self):
        return self.items_for(
            self.referencebrowser_model,
            breakpoint=self.referencable_root
        )

    def item_url(self, node):
        query = make_refbrowser_query(self.request)
        return make_url(self.request, node=node, query=query)

    def item_target(self, node):
        query = make_refbrowser_query(
            self.request, contenttile=node.properties.default_content_tile)
        return make_url(self.request, node=node, query=query)


class ReferenceAction(LinkAction):
    target = None
    href = LinkAction.target

    @property
    def selected_uids(self):
        return [it for it in self.request.params['selected'].split(',') if it]

    @property
    def id(self):
        return 'ref-{}'.format(self.model.uuid)

    @property
    def display(self):
        referencable = self.request.params['referencable']
        referencable = [
            it for it in self.request.params['referencable'].split(',') if it
        ]
        return (
            IUUIDAware.providedBy(self.model) and
            self.model.node_info_name in referencable
        )

    def render(self):
        rendered = LinkAction.render(self)
        attrs = {
            'class_': 'reftitle',
            'style': 'display:none;',
        }
        title = self.model.metadata.get('title', self.model.name)
        return rendered + tag('span', title, **attrs)


class ActionAddReference(ReferenceAction):
    css = 'addreference'
    title = _('add_reference', default='Add reference')
    icon = 'ion-plus-round'

    @property
    def enabled(self):
        if IUUIDAware.providedBy(self.model):
            return not str(self.model.uuid) in self.selected_uids
        return False


class ActionRemoveReference(ReferenceAction):
    css = 'removereference'
    title = _('remove_reference', default='Remove reference')
    icon = 'ion-minus-round'

    @property
    def enabled(self):
        if IUUIDAware.providedBy(self.model):
            return str(self.model.uuid) in self.selected_uids
        return False


class ReferencableChildrenLink(LinkAction):
    event = 'contextchanged:.refbrowsersensitiv'

    def __init__(self, table_tile_name, table_id):
        self.table_tile_name = table_tile_name
        self.table_id = table_id

    @property
    def target(self):
        return '{}{}'.format(
            super(ReferencableChildrenLink, self).target,
            make_refbrowser_query(self.request))

    @property
    def text(self):
        return self.model.metadata.get('title', self.model.name)

    @property
    def action(self):
        return '{}:#{}:replace'.format(self.table_tile_name, self.table_id)

    @property
    def display(self):
        return self.permitted('view')

    @property
    def icon(self):
        return self.model.nodeinfo.icon

    def render(self):
        if INavigationLeaf.providedBy(self.model):
            return u'{}&nbsp;{}'.format(
                tag('span', class_=self.icon),
                tag('span', self.text)
            )
        return LinkAction.render(self)


@tile(name='referencelisting', path='templates/table.pt', permission='view')
class ReferenceListing(ContentsTile):
    table_id = 'referencebrowser'
    table_tile_name = 'referencelisting'
    col_defs = [
        {
            'id': 'actions',
            'title': _('table_actions', default='Actions'),
            'sort_key': None,
            'sort_title': None,
            'content': 'structure',
        },
        {
            'id': 'title',
            'title': _('title', default='Title'),
            'sort_key': None,
            'sort_title': None,
            'content': 'structure',
        },
    ]
    query_whitelist = ['root', 'referencable', 'selected']

    @instance_property
    def row_actions(self):
        row_actions = Toolbar()
        row_actions['add'] = ActionAddReference()
        row_actions['remove'] = ActionRemoveReference()
        return row_actions

    @instance_property
    def referencable_children_link(self):
        return ReferencableChildrenLink(self.table_tile_name, self.table_id)

    def sorted_rows(self, start, end, sort, order):
        children = self.sorted_children(sort, order)
        rows = list()
        for child in children[start:end]:
            row_data = RowData()
            row_data['actions'] = self.row_actions(child, self.request)
            row_data['title'] = \
                self.referencable_children_link(child, self.request)
            rows.append(row_data)
        return rows


def reference_extractor(widget, data):
    if widget.attrs.get('multivalued'):
        return select_extractor(widget, data)
    return data.request.get('{}.uid'.format(widget.dottedpath))


def wrap_ajax_target(rendered, widget, data):
    if widget.attrs.get('target'):
        target = widget.attrs.get('target')
        if callable(target):
            target = target(widget, data)
        referencable = widget.attrs['referencable']
        if callable(referencable):
            referencable = referencable(widget, data)
        if type(referencable) in compat.ITER_TYPES:
            referencable = ','.join(referencable)
        root = widget.attrs['root']
        if callable(root):
            root = root(widget, data)
        selected = ''
        if widget.attrs['multivalued'] and data.value:
            selected = ','.join(data.value)
        elif data.value and data.value[0]:
            selected = data.value[0]
        query = make_query(**{
            'root': root,
            'referencable': referencable,
            'selected': selected,
        })
        target = '{}{}'.format(target, query)
        attrs = {
            'ajax:target': target,
        }
        return tag('span', rendered, **attrs)
    # XXX: referencebrowser not works without target, check if case
    #      necessary. raise error here?
    return rendered


def reference_edit_renderer(widget, data):
    """Properties:

    multivalued
        flag whether reference field is multivalued.

    vocabulary
        if multivalued, provide a vocabulary mapping uids to node names.

    target
        ajax target for reference browser triggering.

    root
        path of reference browser root. Defaults to '/'

    referencable
        list of node info names which are referencable.  Defaults to '',
        which means all objects are referencable, given they provide
        ``IUUIDAware`` and a node info.
    """
    if widget.attrs.get('multivalued'):
        rendered = select_edit_renderer(widget, data)
        return wrap_ajax_target(rendered, widget, data)
    value = ['', '']
    if data.extracted is not UNSET:
        value = [data.extracted, data.request.get(widget.dottedpath)]
    elif data.request.get('{}.uid'.format(widget.dottedpath)):
        value = [
            data.request.get('{}.uid'.format(widget.dottedpath)),
            data.request.get(widget.dottedpath),
        ]
    elif data.value is not UNSET and data.value is not None:
        value = data.value
    text_attrs = {
        'type': 'text',
        'value': value[1],
        'name_': widget.dottedpath,
        'id': cssid(widget, 'input'),
        'class_': cssclasses(widget, data),
        'readonly': 'readonly',
    }
    hidden_attrs = {
        'type': 'hidden',
        'value': value[0],
        'name_': '{}.uid'.format(widget.dottedpath),
    }
    rendered = tag('input', **text_attrs) + tag('input', **hidden_attrs)
    return wrap_ajax_target(rendered, widget, data)


def reference_display_renderer(widget, data):
    if widget.attrs.get('multivalued'):
        return select_display_renderer(widget, data)
    value = fetch_value(widget, data)
    if value in [UNSET, u'', None]:
        value = u''
    else:
        value = value[1]
    attrs = {
        'id': cssid(widget, 'display'),
        'class_': 'display-{}'.format(widget.attrs['class'] or 'generic')
    }
    return data.tag('div', value, **attrs)


factory.register(
    'reference',
    extractors=[generic_extractor, generic_required_extractor,
                reference_extractor],
    edit_renderers=[reference_edit_renderer],
    display_renderers=[reference_display_renderer])

factory.defaults['reference.required_class'] = 'required'

factory.defaults['reference.default'] = ''

factory.defaults['reference.format'] = 'block'

factory.defaults['reference.class'] = 'referencebrowser'

factory.defaults['reference.multivalued'] = False

factory.defaults['reference.root'] = '/'

factory.defaults['reference.referencable'] = ''



from cone.app import security
from cone.app import testing
from cone.app.interfaces import IOwnerSupport
from cone.app.interfaces import IPrincipalACL
from cone.app.model import BaseNode
from cone.app.security import acl_registry
from cone.app.security import authenticate
from cone.app.security import authenticated_user
from cone.app.security import DEFAULT_ACL
from cone.app.security import groups_callback
from cone.app.security import logger
from cone.app.security import OwnerSupport
from cone.app.security import principal_by_id
from cone.app.security import PrincipalACL
from cone.app.security import search_for_principals
from cone.app.ugm import ugm_backend
from node.ext.ugm.interfaces import IGroup
from node.ext.ugm.interfaces import IUser
from node.tests import NodeTestCase
from node.utils import instance_property
from plumber import default
from plumber import plumbing
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.security import ACLAllowed
from pyramid.security import ACLDenied
from pyramid.security import ALL_PERMISSIONS
from pyramid.threadlocal import get_current_registry
from zope.component.globalregistry import BaseGlobalComponents
import logging


class SecurityTest(NodeTestCase):
    layer = testing.security

    def test_superuser_credentials(self):
        # Superuser credentials are set on application startup by main function
        # in cone.app.__init__

        # 'superuser' values get set in test layer
        self.assertEqual(security.ADMIN_USER, 'superuser')
        self.assertEqual(security.ADMIN_PASSWORD, 'superuser')

    def test_authenticated_user(self):
        self.layer.new_request()
        self.assertTrue(authenticated_user(self.layer.current_request) is None)

        with self.layer.authenticated('manager'):
            user = authenticated_user(self.layer.current_request)
            self.assertTrue(IUser.providedBy(user))
            self.assertEqual(user.name, 'manager')

    def test_principal_by_id(self):
        user = principal_by_id('manager')
        self.assertTrue(IUser.providedBy(user))
        self.assertEqual(user.name, 'manager')

        group = principal_by_id('group:group1')
        self.assertTrue(IGroup.providedBy(group))
        self.assertEqual(group.name, 'group1')

        self.assertTrue(principal_by_id('inexistent') is None)

    def test_search_for_principals(self):
        self.assertEqual(search_for_principals('viewer'), [u'viewer'])
        self.assertEqual(search_for_principals('group*'), [u'group:group1'])

    def test_acls(self):
        # The default ACL
        self.assertEqual(security.DEFAULT_ACL, [
            ('Allow', 'system.Authenticated', ['view']),
            ('Allow', 'role:viewer', ['view', 'list']),
            ('Allow', 'role:editor', ['view', 'list', 'add', 'edit']),
            ('Allow', 'role:admin', [
                'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                'paste', 'manage_permissions', 'change_state'
            ]),
            ('Allow', 'role:manager', [
                'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                'paste', 'manage_permissions', 'change_state', 'manage'
            ]),
            ('Allow', 'role:owner', [
                'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                'paste', 'manage_permissions', 'change_state'
            ]),
            ('Allow', 'system.Everyone', ['login']),
            ('Deny', 'system.Everyone', ALL_PERMISSIONS)
        ])

        # Base security tests
        policy = get_current_registry().queryUtility(IAuthenticationPolicy)
        self.assertTrue(isinstance(policy, AuthTktAuthenticationPolicy))

        self.layer.new_request()
        self.assertTrue(isinstance(
            self.layer.current_request.registry,
            BaseGlobalComponents
        ))

        with self.layer.authenticated('inexistent'):
            userid = self.layer.current_request.authenticated_userid
            self.assertTrue(userid is None)

        # Create some security context for testing
        class ACLTest(object):
            __acl__ = DEFAULT_ACL
        context = ACLTest()

        # Authenticate several users and check permission
        with self.layer.authenticated('superuser', 'superuser'):
            userid = self.layer.current_request.authenticated_userid
            self.assertEqual(userid, 'superuser')

            rule = self.layer.current_request.has_permission('manage', context)
            self.assertTrue(isinstance(rule, ACLAllowed))

        with self.layer.authenticated('viewer'):
            userid = self.layer.current_request.authenticated_userid
            self.assertEqual(userid, 'viewer')

            rule = self.layer.current_request.has_permission('manage', context)
            self.assertTrue(isinstance(rule, ACLDenied))

        userid = self.layer.current_request.authenticated_userid
        self.assertTrue(userid is None)

        rule = self.layer.current_request.has_permission('manage', context)
        self.assertTrue(isinstance(rule, ACLDenied))

    def test_ACLRegistry(self):
        class SomeModel(object):
            pass

        acl = [('Allow', 'role:viewer', ['view'])]
        acl_registry.register(acl, SomeModel)

        acl = [('Allow', 'role:viewer', ['edit'])]
        acl_registry.register(acl, node_info_name='some_model')

        acl = [('Allow', 'role:viewer', ['delete'])]
        acl_registry.register(acl, SomeModel, 'some_model')

        self.assertEqual(
            acl_registry.lookup(None, None, [('Allow', 'role:viewer', ['add'])]),
            [('Allow', 'role:viewer', ['add'])]
        )
        self.assertEqual(
            acl_registry.lookup(SomeModel),
            [('Allow', 'role:viewer', ['view'])]
        )
        self.assertEqual(
            acl_registry.lookup(node_info_name='some_model'),
            [('Allow', 'role:viewer', ['edit'])]
        )
        self.assertEqual(
            acl_registry.lookup(SomeModel, 'some_model'),
            [('Allow', 'role:viewer', ['delete'])]
        )

    def test_OwnerSupport(self):
        @plumbing(OwnerSupport)
        class OwnerSupportNode(BaseNode):
            pass

        ownersupportnode = OwnerSupportNode()
        self.assertTrue(IOwnerSupport.providedBy(ownersupportnode))
        self.assertTrue(ownersupportnode.owner is None)
        self.assertEqual(
            ownersupportnode.__acl__[0],
            ('Allow', 'system.Authenticated', ['view'])
        )

        with self.layer.authenticated('sepp'):
            userid = self.layer.current_request.authenticated_userid
            self.assertEqual(userid, 'sepp')

            ownersupportnode = OwnerSupportNode()
            self.assertEqual(ownersupportnode.owner, 'sepp')
            self.assertEqual(ownersupportnode.attrs['owner'], 'sepp')
            self.assertEqual(ownersupportnode.__acl__, [
                ('Allow', 'sepp', [
                    'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                    'paste', 'manage_permissions', 'change_state'
                ]),
                ('Allow', 'system.Authenticated', ['view']),
                ('Allow', 'role:viewer', ['view', 'list']),
                ('Allow', 'role:editor', ['view', 'list', 'add', 'edit']),
                ('Allow', 'role:admin', [
                    'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                    'paste', 'manage_permissions', 'change_state'
                ]),
                ('Allow', 'role:manager', [
                    'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                    'paste', 'manage_permissions', 'change_state', 'manage'
                ]),
                ('Allow', 'role:owner', [
                    'view', 'list', 'add', 'edit', 'delete', 'cut', 'copy',
                    'paste', 'manage_permissions', 'change_state'
                ]),
                ('Allow', 'system.Everyone', ['login']),
                ('Deny', 'system.Everyone', ALL_PERMISSIONS)
            ])

        with self.layer.authenticated('viewer'):
            rule = self.layer.current_request.has_permission(
                'delete',
                ownersupportnode
            )
            self.assertTrue(isinstance(rule, ACLDenied))

        with self.layer.authenticated('sepp'):
            rule = self.layer.current_request.has_permission(
                'delete',
                ownersupportnode
            )
            self.assertTrue(isinstance(rule, ACLAllowed))

        @plumbing(OwnerSupport)
        class NoOwnerACLOnBaseNode(BaseNode):
            @property
            def __acl__(self):
                return [('Allow', 'role:viewer', ['view'])]

        with self.layer.authenticated('sepp'):
            ownersupportnode = NoOwnerACLOnBaseNode()
            self.assertEqual(ownersupportnode.owner, 'sepp')
            self.assertEqual(
                ownersupportnode.__acl__,
                [('Allow', 'role:viewer', ['view'])]
            )

    def test_PrincipalACL(self):
        # PrincipalACL is an abstract class. Directly use causes an error
        @plumbing(PrincipalACL)
        class PrincipalACLNode(BaseNode):
            pass

        node = PrincipalACLNode()
        err = self.expect_error(NotImplementedError, lambda: node.__acl__)
        expected = (
            'Abstract ``PrincipalACL`` does not implement ``principal_roles``.'
        )
        self.assertEqual(str(err), expected)

        # Concrete PrincipalACL implementation. Implements principal_roles
        # property
        class MyPrincipalACL(PrincipalACL):
            @default
            @instance_property
            def principal_roles(self):
                return dict()

        @plumbing(MyPrincipalACL)
        class MyPrincipalACLNode(BaseNode):
            pass

        node = MyPrincipalACLNode()
        self.assertTrue(IPrincipalACL.providedBy(node))

        node.principal_roles['someuser'] = ['manager']
        node.principal_roles['otheruser'] = ['editor']
        node.principal_roles['group:some_group'] = ['editor', 'manager']

        def find_rule(acl, who):
            for rule in acl:
                if rule[1] == who:
                    return rule

        rule = find_rule(node.__acl__, 'someuser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'someuser')
        self.assertEqual(sorted(rule[2]), sorted([
            'cut', 'edit', 'copy', 'manage', 'list', 'add', 'change_state',
            'view', 'paste', 'manage_permissions', 'delete'
        ]))

        rule = find_rule(node.__acl__, 'otheruser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'otheruser')
        self.assertEqual(sorted(rule[2]), sorted([
            'edit', 'add', 'list', 'view'
        ]))

        rule = find_rule(node.__acl__, 'group:some_group')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'group:some_group')
        self.assertEqual(sorted(rule[2]), sorted([
            'cut', 'edit', 'copy', 'manage', 'list', 'add', 'change_state',
            'view', 'paste', 'manage_permissions', 'delete'
        ]))

        rule = find_rule(node.__acl__, 'system.Authenticated')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'system.Authenticated')
        self.assertEqual(rule[2], ['view'])

        rule = find_rule(node.__acl__, 'role:viewer')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'role:viewer')
        self.assertEqual(sorted(rule[2]), sorted(['view', 'list']))

        rule = node.__acl__[-1]
        self.assertEqual(rule[0], 'Deny')
        self.assertEqual(rule[1], 'system.Everyone')
        self.assertEqual(rule[2], ALL_PERMISSIONS)

        # PrincipalACL role inheritance
        child = node['child'] = MyPrincipalACLNode()
        child.principal_roles['someuser'] = ['editor']

        rule = find_rule(child.__acl__, 'someuser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'someuser')
        self.assertEqual(sorted(rule[2]), sorted([
            'edit', 'add', 'list', 'view'
        ]))

        rule = find_rule(child.__acl__, 'system.Authenticated')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'system.Authenticated')
        self.assertEqual(rule[2], ['view'])

        rule = find_rule(child.__acl__, 'role:viewer')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'role:viewer')
        self.assertEqual(sorted(rule[2]), sorted(['view', 'list']))

        rule = child.__acl__[-1]
        self.assertEqual(rule[0], 'Deny')
        self.assertEqual(rule[1], 'system.Everyone')
        self.assertEqual(rule[2], ALL_PERMISSIONS)

        subchild = child['child'] = MyPrincipalACLNode()
        subchild.role_inheritance = True
        subchild.principal_roles['otheruser'] = ['admin']
        self.assertEqual(subchild.aggregated_roles_for('inexistent'), [])
        self.assertEqual(
            sorted(subchild.aggregated_roles_for('someuser')),
            sorted(['manager', 'editor'])
        )
        self.assertEqual(
            sorted(subchild.aggregated_roles_for('otheruser')),
            sorted(['admin', 'editor'])
        )

        rule = find_rule(subchild.__acl__, 'someuser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'someuser')
        self.assertEqual(sorted(rule[2]), sorted([
            'cut', 'edit', 'copy', 'manage', 'list', 'add', 'change_state',
            'view', 'paste', 'manage_permissions', 'delete'
        ]))

        rule = find_rule(subchild.__acl__, 'otheruser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'otheruser')
        self.assertEqual(sorted(rule[2]), sorted([
            'cut', 'edit', 'copy', 'list', 'add', 'change_state', 'view',
            'paste', 'manage_permissions', 'delete'
        ]))

        rule = find_rule(subchild.__acl__, 'group:some_group')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'group:some_group')
        self.assertEqual(sorted(rule[2]), sorted([
            'cut', 'edit', 'copy', 'manage', 'list', 'add', 'change_state',
            'view', 'paste', 'manage_permissions', 'delete'
        ]))

        rule = find_rule(subchild.__acl__, 'system.Authenticated')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'system.Authenticated')
        self.assertEqual(rule[2], ['view'])

        rule = subchild.__acl__[-1]
        self.assertEqual(rule[0], 'Deny')
        self.assertEqual(rule[1], 'system.Everyone')
        self.assertEqual(rule[2], ALL_PERMISSIONS)

        # Principal roles get inherited even if some parent does not provide
        # principal roles
        child = node['no_principal_roles'] = BaseNode()
        subchild = child['no_principal_roles'] = MyPrincipalACLNode()
        self.assertEqual(
            sorted(subchild.aggregated_roles_for('group:some_group')),
            sorted(['manager', 'editor'])
        )

        # If principal role found which is not provided by plumbing endpoint
        # acl, this role does not grant any permissions
        node = MyPrincipalACLNode()
        node.principal_roles['someuser'] = ['inexistent_role']

        rule = find_rule(node.__acl__, 'someuser')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'someuser')
        self.assertEqual(rule[2], [])

        rule = find_rule(node.__acl__, 'system.Authenticated')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'system.Authenticated')
        self.assertEqual(rule[2], ['view'])

        rule = find_rule(node.__acl__, 'role:viewer')
        self.assertEqual(rule[0], 'Allow')
        self.assertEqual(rule[1], 'role:viewer')
        self.assertEqual(sorted(rule[2]), sorted(['view', 'list']))

        rule = node.__acl__[-1]
        self.assertEqual(rule[0], 'Deny')
        self.assertEqual(rule[1], 'system.Everyone')
        self.assertEqual(rule[2], ALL_PERMISSIONS)
        self.assertEqual(
            node.__acl__[-1],
            ('Deny', 'system.Everyone', ALL_PERMISSIONS)
        )

    def test_authentication_logging(self):
        # If an authentication plugin raises an error when calling
        # ``authenticate``, an error message is logged
        class TestHandler(logging.StreamHandler):
            record = None

            def handle(self, record):
                self.record = record

        handler = TestHandler()

        logger.addHandler(handler)
        logger.setLevel(logging.DEBUG)

        orgin_ugm = ugm_backend.ugm
        ugm_backend.ugm = object()

        authenticate(self.layer.new_request(), 'foo', 'foo')

        self.check_output("""
        <LogRecord: cone.app, ..., ...security.py, ...,
        "Authentication plugin <... 'object'> raised an Exception while trying
        to authenticate: 'object' object has no attribute 'users'">
        """, str(handler.record))

        # Test Group callback, also logs if an error occurs
        with self.layer.authenticated('superuser', 'superuser'):
            self.assertEqual(
                groups_callback('superuser', self.layer.new_request()),
                [u'role:manager']
            )

        groups_callback('foo', self.layer.new_request())
        self.check_output("""
        <LogRecord: cone.app, 40,
        ...security.py, ..., "'object' object has no attribute 'users'">
        """, str(handler.record))

        # Cleanup
        logger.setLevel(logging.INFO)
        logger.removeHandler(handler)
        ugm_backend.ugm = orgin_ugm



from cone.app.browser import RelatedViewProvider
from cone.app.browser import get_related_view
from cone.app.browser import render_main_template
from cone.app.browser.actions import ActionDelete
from cone.app.browser.actions import ActionEdit
from cone.app.browser.actions import ActionView
from cone.app.browser.actions import Toolbar
from cone.app.browser.actions import ViewLink
from cone.app.browser.copysupport import extract_copysupport_cookie
from cone.app.browser.table import RowData
from cone.app.browser.table import Table
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.interfaces import ICopySupport
from cone.app.interfaces import IWorkflowState
from cone.tile import Tile
from cone.tile import tile
from node.interfaces import ILeaf
from node.utils import instance_property
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
from pyramid.view import view_config
import datetime


_ = TranslationStringFactory('cone.app')


FAR_PAST = datetime.datetime(2000, 1, 1)


class ContentsActionView(ActionView):
    title = ActionView.text
    text = None
    action = None
    event = 'contextchanged:#layout'


class ContentsActionEdit(ActionEdit):
    title = ActionEdit.text
    text = None
    action = None
    event = 'contextchanged:#layout'

    @property
    def target(self):
        query = make_query(contenttile='edit')
        return make_url(self.request, node=self.model, query=query)


class ContentsActionDelete(ActionDelete):
    """Delete action for contents table.
    """
    title = ActionDelete.text
    text = None

    @property
    def display(self):
        return self.model.properties.action_delete \
            and self.request.has_permission('delete', self.model.parent) \
            and self.permitted('delete')


class ContentsViewLink(ViewLink):
    """View link for contents table.
    """
    css = 'title'
    event = 'contextchanged:#layout'
    action = None

    @property
    def target(self):
        related_view = get_related_view(self.request)
        if related_view and not ILeaf.providedBy(self.model):
            query = make_query(contenttile=related_view)
            return make_url(self.request, node=self.model, query=query)
        return super(ContentsViewLink, self).target


@tile(name='contents', path='templates/table.pt', permission='list')
class ContentsTile(Table):
    table_id = 'contents'
    table_tile_name = 'contents'
    col_defs = [{
        'id': 'actions',
        'title': _('actions', default='Actions'),
        'sort_key': None,
        'sort_title': None,
        'content': 'structure'
    }, {
        'id': 'title',
        'title': _('title', default='Title'),
        'sort_key': 'title',
        'sort_title': _('sort_on_title', default='Sort on title'),
        'content': 'structure'
    }, {
        'id': 'creator',
        'title': _('creator', default='Creator'),
        'sort_key': 'creator',
        'sort_title': _('sort_on_creator', default='Sort on creator'),
        'content': 'string'
    }, {
        'id': 'created',
        'title': _('created', default='Created'),
        'sort_key': 'created',
        'sort_title': _('sort_on_created', default='Sort on created'),
        'content': 'datetime'
    }, {
        'id': 'modified',
        'title': _('modified', default='Modified'),
        'sort_key': 'modified',
        'sort_title': _('sort_on_modified', default='Sort on modified'),
        'content': 'datetime'
    }]
    default_sort = 'created'
    default_order = 'desc'
    sort_keys = {
        'title': lambda x: x.metadata.title.lower(),
        'creator': lambda x: x.metadata.creator.lower(),
        'created': lambda x: (
            x.metadata.created and x.metadata.created or FAR_PAST
        ),
        'modified': lambda x: (
            x.metadata.modified and x.metadata.modified or FAR_PAST
        )
    }
    show_filter = True

    @property
    def item_count(self):
        return len(self.filtered_children)

    @instance_property
    def row_actions(self):
        row_actions = Toolbar()
        row_actions['view'] = ContentsActionView()
        row_actions['edit'] = ContentsActionEdit()
        row_actions['delete'] = ContentsActionDelete()
        return row_actions

    @instance_property
    def view_link(self):
        return ContentsViewLink()

    def row_data(self, node):
        row_data = RowData()
        row_data['actions'] = self.row_actions(node, self.request)
        row_data['title'] = self.view_link(node, self.request)
        row_data['creator'] = node.metadata.get('creator', 'unknown')
        row_data['created'] = node.metadata.get('created')
        row_data['modified'] = node.metadata.get('modified')
        return row_data

    def sorted_rows(self, start, end, sort, order):
        children = self.sorted_children(sort, order)
        rows = list()
        cut_urls = extract_copysupport_cookie(self.request, 'cut')
        for child in children[start:end]:
            row_data = self.row_data(child)
            target = make_url(self.request, node=child)
            if ICopySupport.providedBy(child):
                row_data.selectable = True
                row_data.target = target
                row_data.css = 'copysupportitem'
                if target in cut_urls:
                    row_data.css += ' copysupport_cut'
            if IWorkflowState.providedBy(child):
                row_data.css += ' state-%s' % child.state
            if hasattr(child, 'node_info_name') and child.node_info_name:
                row_data.css += ' node-type-%s' % child.node_info_name
            rows.append(row_data)
        return rows

    @property
    def listable_children(self):
        return self.model.values()

    @property
    def filtered_children(self):
        if '_filtered_children' in self.request.environ:
            return self.request.environ['_filtered_children']
        children = list()
        term = self.filter_term
        if term:
            term = term.lower()
        for node in self.listable_children:
            if not self.request.has_permission('view', node):
                continue
            if term:
                metadata = node.metadata
                title = metadata.get('title')
                title = title.lower() if title else ''
                creator = metadata.get('creator')
                creator = creator.lower() if creator else ''
                if title.find(term) == -1 and creator.find(term) == -1:
                    continue
            children.append(node)
        self.request.environ['_filtered_children'] = children
        return children

    def sorted_children(self, sort, order):
        children = self.filtered_children
        if sort in self.sort_keys:
            children = sorted(children, key=self.sort_keys[sort])
            if order == 'asc':
                children.reverse()
        return children


@tile(name='listing', path='templates/listing.pt', permission='list')
@plumbing(RelatedViewProvider)
class ListingTile(Tile):
    """Tile rendering the listing.
    """
    related_view = 'listing'


@view_config(name='listing', permission='list')
def listing(model, request):
    """Listing view.
    """
    return render_main_template(model, request, 'listing')



from cone.app import model
from cone.app.security import authenticate
from cone.tile.tests import DummyVenusian
from contextlib import contextmanager
from pyramid.security import AuthenticationAPIMixin
from pyramid.testing import DummyRequest as BaseDummyRequest
from zope.component import getGlobalSiteManager
from zope.component.hooks import resetHooks
from zope.configuration.xmlconfig import XMLConfig
import cone.app
import cone.tile
import os
import venusian


def reset_node_info_registry(fn):
    """Decorator for tests using node info registry
    """
    def wrapper(*a, **kw):
        try:
            fn(*a, **kw)
        finally:
            model._node_info_registry = dict()
    return wrapper


class DummyRequest(BaseDummyRequest, AuthenticationAPIMixin):

    @property
    def is_xhr(self):
        return self.headers.get('X-Requested-With') == 'XMLHttpRequest'


DATADIR = os.path.join(os.path.dirname(__file__), 'data', 'ugm')


class Security(object):
    """Test layer with dummy authentication for security testing.
    """
    current_request = None
    auth_env_keys = [
        'REMOTE_USER_TOKENS',
        'REMOTE_USER_DATA',
        'HTTP_COOKIE',
        'cone.app.user.roles',
    ]

    def __init__(self):
        self.__name__ = self.__class__.__name__
        self.__bases__ = []

    @property
    def registry(self):
        return getGlobalSiteManager()

    def defaults(self):
        return {
            'request': self.current_request,
            'registry': self.registry
        }

    def new_request(self, type=None, xhr=False):
        request = self.current_request
        auth = dict()
        cookies = dict()
        if request:
            cookies = request.cookies
            environ = request.environ
            for key in self.auth_env_keys:
                if key in environ:
                    auth[key] = environ[key]
        request = DummyRequest()
        request.cookies.update(cookies)
        request.environ['SERVER_NAME'] = 'testcase'
        request.environ['AUTH_TYPE'] = 'cookie'
        request.environ.update(auth)
        request.params['_LOCALE_'] = 'en'
        if type == 'json':
            request.headers['X-Request'] = 'JSON'
            request.accept = 'application/json'
        if xhr:
            request.headers['X-Requested-With'] = 'XMLHttpRequest'
        self.current_request = request
        return request

    def login(self, login, password=None):
        request = self.current_request
        if not request:
            request = self.new_request()
        else:
            self.logout()
        password = password if password else 'secret'
        res = authenticate(request, login, password)
        if res:
            request.environ['HTTP_COOKIE'] = res[0][1]
            cookie = res[0][1].split(';')[0].split('=')
            request.cookies[cookie[0]] = cookie[1]

    def logout(self):
        request = self.current_request
        if request:
            request.cookies.clear()
            environ = request.environ
            for key in self.auth_env_keys:
                if key in environ:
                    del environ[key]

    @contextmanager
    def authenticated(self, login, password=None):
        try:
            self.login(login, password=password)
            yield
        finally:
            self.logout()

    @contextmanager
    def hook_tile_reg(self):
        try:
            cone.tile.tile.venusian = DummyVenusian()
            yield
        finally:
            cone.tile.tile.venusian = venusian

    def make_app(self, **kw):
        import pyramid.threadlocal
        pyramid.threadlocal.manager.default = self.defaults
        settings = {
            'default_locale_name': 'en',
            'cone.admin_user': 'superuser',
            'cone.admin_password': 'superuser',
            'cone.auth_secret': '12345',
            'cone.plugins': 'node.ext.ugm\ninexistent',
            'cone.root.title': 'cone',
            'cone.root.default_child': None,
            'cone.root.default_content_tile': 'content',
            'cone.root.mainmenu_empty_title': False,
            'ugm.backend': 'file',
            'ugm.users_file': os.path.join(DATADIR, 'users'),
            'ugm.groups_file': os.path.join(DATADIR, 'groups'),
            'ugm.roles_file': os.path.join(DATADIR, 'roles'),
            'ugm.datadir': os.path.join(DATADIR, 'userdata'),
            'testing.hook_global_registry': True,
        }
        settings.update(**kw)
        self.app = cone.app.main({}, **settings)
        self.current_request = None

    def setUp(self, args=None):
        self.make_app()
        XMLConfig('testing/dummy_workflow.zcml', cone.app)()
        print("Security set up.")

    def tearDown(self):
        # XXX: something is wrong here.
        import pyramid.threadlocal
        pyramid.threadlocal.manager.default = pyramid.threadlocal.defaults
        resetHooks()
        print("Security torn down.")


security = Security()



# -*- coding: utf-8 -*-
from cone.app import browser
from cone.app import security
from cone.app.interfaces import ILayout
from cone.app.model import AppRoot
from cone.app.model import AppSettings
from cone.app.model import Layout
from cone.app.model import Properties
from cone.app.ugm import ugm_backend
from cone.app.utils import format_traceback
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.config import Configurator
from pyramid.static import static_view
from yafowil.resources import YafowilResources as YafowilResourcesBase
from zope.component import adapter
from zope.component import getGlobalSiteManager
from zope.interface import implementer
from zope.interface import Interface
import importlib
import logging
import pyramid_chameleon
import pyramid_zcml
import sys


logger = logging.getLogger('cone.app')

# configuration
cfg = Properties()

# used main template
cfg.main_template = 'cone.app.browser:templates/main.pt'

# default node icon
cfg.default_node_icon = 'glyphicon glyphicon-asterisk'

# XXX: move resource registration to browser package
# XXX: support developmenet and production mode

# JS resources
cfg.js = Properties()
cfg.js.public = [
    '++resource++bdajax/overlay.js',
    '++resource++bdajax/bdajax.js',
    '++resource++bdajax/bdajax_bs3.js',
    'static/public.js'
]
cfg.js.protected = [
    'static/protected.js'
]

# CSS Resources
cfg.css = Properties()

# development
cfg.css.public = [
    'static/jqueryui/jquery-ui-1.10.3.custom.css',
    'static/bootstrap/css/bootstrap.css',
    'static/bootstrap/css/bootstrap-theme.css',
    'static/ionicons/css/ionicons.css',
    'static/typeahead/typeahead.css',
    '++resource++bdajax/bdajax_bs3.css',
    'static/styles.css'
]

# production
# cfg.css.public = [
#     'static/jqueryui/jquery-ui-1.10.3.custom.css',
#     'static/bootstrap/css/bootstrap.min.css',
#     'static/bootstrap/css/bootstrap-theme.min.css',
#     'static/ionicons/css/ionicons.css',
#     'static/typeahead/typeahead.css',
#     '++resource++bdajax/bdajax_bs3.css',
#     'static/styles.css'
# ]

cfg.css.protected = list()

# JS and CSS Assets to publish merged
cfg.merged = Properties()
cfg.merged.js = Properties()

# development
cfg.merged.js.public = [
    (browser.static_resources, 'jquery-1.9.1.js'),
    (browser.static_resources, 'jquery.migrate-1.2.1.js'),
    (browser.static_resources, 'jqueryui/jquery-ui-1.10.3.custom.js'),
    (browser.static_resources, 'bootstrap/js/bootstrap.js'),
    (browser.static_resources, 'typeahead/typeahead.bundle.js'),
    (browser.static_resources, 'cookie_functions.js')
]

# production
# cfg.merged.js.public = [
#     (browser.static_resources, 'jquery-1.9.1.min.js'),
#     (browser.static_resources, 'jquery.migrate-1.2.1.min.js'),
#     (browser.static_resources, 'jqueryui/jquery-ui-1.10.3.custom.min.js'),
#     (browser.static_resources, 'bootstrap/js/bootstrap.min.js'),
#     (browser.static_resources, 'typeahead/typeahead.bundle.js'),
#     (browser.static_resources, 'cookie_functions.js')
# ]

cfg.merged.js.protected = list()

cfg.merged.css = Properties()
cfg.merged.css.public = list()
cfg.merged.css.protected = list()

cfg.merged.print_css = Properties()
cfg.merged.print_css.public = [
    (browser.static_resources, 'print.css')
]
cfg.merged.print_css.protected = list()

# root node
root = AppRoot()
root.factories['settings'] = AppSettings


@implementer(ILayout)
@adapter(Interface)
def default_layout(context):
    layout = Layout()
    layout.mainmenu = True
    layout.mainmenu_fluid = False
    layout.livesearch = True
    layout.personaltools = True
    layout.columns_fluid = False
    layout.pathbar = True
    layout.sidebar_left = ['navtree']
    layout.sidebar_left_grid_width = 3
    layout.content_grid_width = 9
    return layout


def configure_root(settings):
    root.metadata.title = settings.get('cone.root.title', 'CONE')
    root.properties.default_child = settings.get('cone.root.default_child')
    root.properties.mainmenu_empty_title = \
        settings.get('cone.root.mainmenu_empty_title', 'false') \
        in ['True', 'true', '1']
    default_content_tile = settings.get('cone.root.default_content_tile')
    if default_content_tile:
        root.properties.default_content_tile = default_content_tile
    root.properties.in_navtree = False


def register_config(key, factory):
    factories = root['settings'].factories
    if key in factories:
        raise ValueError(u"Config with name '%s' already registered." % key)
    factories[key] = factory


# B/C
register_plugin_config = register_config


def register_entry(key, factory):
    factories = root.factories
    if key in factories:
        raise ValueError(u"Entry with name '%s' already registered." % key)
    root.factories[key] = factory


# B/C
register_plugin = register_entry


main_hooks = list()


def main_hook(func):
    """decorator to register main hook.

    Decorated function gets called on application startup.
    """
    main_hooks.append(func)
    return func


# B/C
def register_main_hook(callback):
    """Register function to get called on application startup.
    """
    main_hooks.append(callback)


def get_root(environ=None):
    return root


def auth_tkt_factory(**kwargs):
    kwargs.setdefault('callback', security.groups_callback)
    return AuthTktAuthenticationPolicy(**kwargs)


def acl_factory(**kwargs):
    return ACLAuthorizationPolicy()


cfg.yafowil = Properties()
cfg.yafowil.js_skip = set()
cfg.yafowil.css_skip = set()

# ignore bootstrap dependencies delivered by yafowil.bootstrap
cfg.yafowil.js_skip.add('bootstrap.dependencies')
cfg.yafowil.css_skip.add('bootstrap.dependencies')


class YafowilResources(YafowilResourcesBase):

    def __init__(self, js_skip=[], css_skip=[], config=None):
        self.config = config
        super(YafowilResources, self).__init__(
            js_skip=js_skip,
            css_skip=css_skip
        )

    def configure_resource_directory(self, plugin_name, resourc_edir):
        app = sys.modules[__name__]
        resources_view = static_view(resourc_edir, use_subpath=True)
        view_name = '%s_resources' % plugin_name.replace('.', '_')
        setattr(app, view_name, resources_view)
        view_path = 'cone.app.%s' % view_name
        resource_base = '++resource++%s' % plugin_name
        self.config.add_view(view_path, name=resource_base)
        return resource_base


def configure_yafowil_addon_resources(config):
    resources = YafowilResources(
        js_skip=cfg.yafowil.js_skip,
        css_skip=cfg.yafowil.css_skip,
        config=config
    )
    for js in reversed(resources.js_resources):
        # bdajax needs to be loaded first in order to avoid double binding on
        # document ready
        idx = cfg.js.public.index('++resource++bdajax/bdajax.js') + 1
        cfg.js.public.insert(idx, js)
    for css in resources.css_resources:
        cfg.css.public.insert(0, css)


def main(global_config, **settings):
    """Returns WSGI application.
    """
    # set authentication related application properties
    security.ADMIN_USER = settings.get('cone.admin_user')
    security.ADMIN_PASSWORD = settings.get('cone.admin_password')

    auth_secret = settings.pop('cone.auth_secret', 'secret')
    auth_cookie_name = settings.pop('cone.auth_cookie_name', 'auth_tkt')
    auth_secure = settings.pop('cone.auth_secure', False)
    auth_include_ip = settings.pop('cone.auth_include_ip', False)
    auth_timeout = settings.pop('cone.auth_timeout', None)
    auth_reissue_time = settings.pop('cone.auth_reissue_time', None)
    if auth_reissue_time is not None:
        auth_reissue_time = int(auth_reissue_time)
    auth_max_age = settings.pop('cone.auth_max_age', None)
    if auth_max_age is not None:
        auth_max_age = int(auth_max_age)
    auth_http_only = settings.pop('cone.auth_http_only', False)
    auth_path = settings.pop('cone.auth_path', "/")
    auth_wild_domain = settings.pop('cone.auth_wild_domain', True)

    auth_policy = auth_tkt_factory(
        secret=auth_secret,
        cookie_name=auth_cookie_name,
        secure=auth_secure,
        include_ip=auth_include_ip,
        timeout=auth_timeout,
        reissue_time=auth_reissue_time,
        max_age=auth_max_age,
        http_only=auth_http_only,
        path=auth_path,
        wild_domain=auth_wild_domain,
    )

    configure_root(settings)

    if settings.get('testing.hook_global_registry'):
        globalreg = getGlobalSiteManager()
        config = Configurator(registry=globalreg)
        config.setup_registry(root_factory=get_root, settings=settings)
    else:
        config = Configurator(root_factory=get_root, settings=settings)

    # set authentication and authorization policies
    config.set_authentication_policy(auth_policy)
    config.set_authorization_policy(acl_factory())
    config.commit()

    # begin configuration
    config.begin()

    # include general dependencies
    config.include(pyramid_chameleon)
    config.include(pyramid_zcml)

    # register default layout adapter
    config.registry.registerAdapter(default_layout)

    # add translation
    config.add_translation_dirs('cone.app:locale/')

    # XXX: register yafowil and all yafowil addon widget locales.
    #      provide locales either in yafowil resources or as entry points in
    #      all yafowil packages providing translations

    # static routes
    config.add_route("favicon", "/favicon.ico")
    # XXX: robots.txt
    # XXX: humans.txt

    # register static resources
    config.add_view(browser.static_resources, name='static')

    # scan browser package
    config.scan(browser)

    # load zcml
    config.load_zcml('configure.zcml')

    # read plugin configurator
    plugins = settings.get('cone.plugins', '')
    plugins = plugins.split('\n')
    plugins = [pl for pl in plugins if pl and not pl.startswith('#')]
    for plugin in plugins:
        try:
            importlib.import_module(plugin)
        except ImportError:
            msg = 'Cannot import plugin {}\n{}'.format(
                plugin,
                format_traceback()
            )
            logger.error(msg)
            continue
        try:
            config.load_zcml('{}:configure.zcml'.format(plugin))
        except IOError:  # pragma: no cover
            msg = 'No configure.zcml in {}'.format(plugin)
            logger.info(msg)

    # execute main hooks
    filtered_hooks = list()
    for hook in main_hooks:
        for plugin in plugins:
            if hook.__module__.startswith(plugin):
                filtered_hooks.append(hook)
                continue
    for hook in filtered_hooks:
        hook(config, global_config, settings)

    # load and initialize UGM
    backend_name = settings.get('ugm.backend')
    # B/C
    if not backend_name:
        backend_name = settings.get('cone.auth_impl')
    if backend_name:
        try:
            ugm_backend.load(backend_name, settings)
            ugm_backend.initialize()
        except Exception:  # pragma: no cover
            msg = 'Failed to create UGM backend:\n{}'.format(format_traceback())
            logger.error(msg)

    # register yafowil static resources
    # done after addon config - addon code may disable yafowil resource groups
    configure_yafowil_addon_resources(config)

    # end configuration
    config.end()

    # return wsgi app
    return config.make_wsgi_app()


def make_remote_addr_middleware(app, global_conf):
    return RemoteAddrFilter(app)


class RemoteAddrFilter(object):
    """Use this middleware if nginx is used as proxy and IP address should be
    included in auth cookie. make sure nginx passes the right header:

    proxy_set_header X-Real-IP $remote_addr;
    """

    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        if 'HTTP_X_REAL_IP' in environ:
            environ['REMOTE_ADDR'] = environ['HTTP_X_REAL_IP']
        return self.app(environ, start_response)
